@@ -34,16 +34,17 @@ import (
 )
 
 const (
-	projectId      = "project1"
-	zoneB          = "us-central1-b"
-	zoneC          = "us-central1-c"
-	zoneF          = "us-central1-f"
-	region         = "us-central1"
-	defaultPoolMig = "gke-cluster-1-default-pool"
-	defaultPool    = "default-pool"
-	extraPoolMig   = "gke-cluster-1-extra-pool-323233232"
-	extraPool      = "extra-pool"
-	clusterName    = "cluster1"
+	projectId          = "project1"
+	zoneB              = "us-central1-b"
+	zoneC              = "us-central1-c"
+	zoneF              = "us-central1-f"
+	region             = "us-central1"
+	defaultPoolMigName = "gke-cluster-1-default-pool"
+	defaultPool        = "default-pool"
+	extraPoolMigName   = "gke-cluster-1-extra-pool-323233232"
+	extraPool2MigName  = "gke-cluster-1-extra-pool2-323233232"
+	extraPool          = "extra-pool"
+	clusterName        = "cluster1"
 
 	gceMigA = "gce-mig-a"
 	gceMigB = "gce-mig-b"
@@ -57,7 +58,7 @@ const instanceGroupManagerResponseTemplate = `{
   "zone": "https://www.googleapis.com/compute/v1/projects/project1/zones/%s",
   "instanceTemplate": "https://www.googleapis.com/compute/v1/projects/project1/global/instanceTemplates/%s",
   "instanceGroup": "https://www.googleapis.com/compute/v1/projects/project1/zones/%s/instanceGroups/%s",
-  "baseInstanceName": "gke-cluster-1-default-pool-f23aac-grp",
+  "baseInstanceName": "%s",
   "fingerprint": "kfdsuH",
   "currentActions": {
     "none": 3,
@@ -69,7 +70,7 @@ const instanceGroupManagerResponseTemplate = `{
     "restarting": 0,
     "refreshing": 0
   },
-  "targetSize": 3,
+  "targetSize": %v,
   "selfLink": "https://www.googleapis.com/compute/v1/projects/project1/zones/%s/instanceGroupManagers/%s"
 }
 `
@@ -236,16 +237,66 @@ const oneRunningInstanceManagedInstancesResponseTemplate = `{
   ]
 }`
 
+const listInstanceGroupManagerResponsePartTemplate = `
+  {
+   "kind": "compute#instanceGroupManager",
+   "id": "9012769713544464023",
+   "creationTimestamp": "2019-03-26T07:34:32.082-07:00",
+   "name": "%v",
+   "zone": "https://www.googleapis.com/compute/v1/projects/project1/zones/%v",
+   "instanceTemplate": "https://www.googleapis.com/compute/v1/projects/project1/global/instanceTemplates/gke-blah-default-pool-67b773a0",
+   "versions": [
+    {
+     "instanceTemplate": "https://www.googleapis.com/compute/v1/projects/project1/global/instanceTemplates/gke-blah-default-pool-67b773a0",
+     "targetSize": {
+      "calculated": 1
+     }
+    }
+   ],
+   "instanceGroup": "https://www.googleapis.com/compute/v1/projects/lukaszos-gke-dev2/zones/%v/instanceGroups/%v",
+   "baseInstanceName": "gke-blah-default-pool-67b773a0",
+   "fingerprint": "ASJwTpesjDI=",
+   "currentActions": {
+    "none": 1,
+    "creating": 0,
+    "creatingWithoutRetries": 0,
+    "verifying": 0,
+    "recreating": 0,
+    "deleting": 0,
+    "abandoning": 0,
+    "restarting": 0,
+    "refreshing": 0
+   },
+   "status": {
+    "isStable": true
+   },
+   "targetSize": %v,
+   "selfLink": "https://www.googleapis.com/compute/v1/projects/lukaszos-gke-dev2/zones/us-west1-b/instanceGroupManagers/gke-blah-default-pool-67b773a0-grp",
+   "updatePolicy": {
+    "type": "OPPORTUNISTIC",
+    "minimalAction": "REPLACE",
+    "maxSurge": {
+     "fixed": 1,
+     "calculated": 1
+    },
+    "maxUnavailable": {
+     "fixed": 1,
+     "calculated": 1
+    }
+   }
+  }
+`
+
 func buildDefaultInstanceGroupManagerResponse(zone string) string {
-	return buildInstanceGroupManagerResponse(zone, defaultPoolMig)
+	return buildInstanceGroupManagerResponse(zone, defaultPoolMigName, 3)
 }
 
-func buildInstanceGroupManagerResponse(zone string, instanceGroup string) string {
-	return fmt.Sprintf(instanceGroupManagerResponseTemplate, instanceGroup, zone, instanceGroup, zone, instanceGroup, zone, instanceGroup)
+func buildInstanceGroupManagerResponse(zone string, instanceGroup string, targetSize uint64) string {
+	return fmt.Sprintf(instanceGroupManagerResponseTemplate, instanceGroup, zone, instanceGroup, zone, instanceGroup, instanceGroup, targetSize, zone, instanceGroup)
 }
 
 func buildFourRunningInstancesOnDefaultMigManagedInstancesResponse(zone string) string {
-	return buildFourRunningInstancesManagedInstancesResponse(zone, defaultPoolMig)
+	return buildFourRunningInstancesManagedInstancesResponse(zone, defaultPoolMigName)
 }
 
 func buildFourRunningInstancesManagedInstancesResponse(zone string, instanceGroup string) string {
@@ -253,13 +304,26 @@ func buildFourRunningInstancesManagedInstancesResponse(zone string, instanceGrou
 }
 
 func buildOneRunningInstanceOnExtraPoolMigManagedInstancesResponse(zone string) string {
-	return buildOneRunningInstanceManagedInstancesResponse(zone, extraPoolMig)
+	return buildOneRunningInstanceManagedInstancesResponse(zone, extraPoolMigName)
 }
 
 func buildOneRunningInstanceManagedInstancesResponse(zone string, instanceGroup string) string {
 	return fmt.Sprintf(oneRunningInstanceManagedInstancesResponseTemplate, zone, instanceGroup)
 }
 
+func buildListInstanceGroupManagersResponsePart(name, zone string, targetSize uint64) string {
+	return fmt.Sprintf(listInstanceGroupManagerResponsePartTemplate, name, zone, zone, name, targetSize)
+}
+
+func buildListInstanceGroupManagersResponse(listInstanceGroupManagerResponseParts ...string) string {
+	return `{
+ "kind": "compute#instanceGroupManagerList",
+ "id": "blah",
+ "items": [` +
+		strings.Join(listInstanceGroupManagerResponseParts, ",") +
+		`], "selfLink": "https://blah"}`
+}
+
 func newTestGceManager(t *testing.T, testServerURL string, regional bool) *gceManagerImpl {
 	gceService := newTestAutoscalingGceClient(t, projectId, testServerURL)
 
@@ -267,23 +331,28 @@ func newTestGceManager(t *testing.T, testServerURL string, regional bool) *gceMa
 	gceService.operationWaitTimeout = 50 * time.Millisecond
 	gceService.operationPollInterval = 1 * time.Millisecond
 
-	manager := &gceManagerImpl{
-		cache: GceCache{
-			migs:           make([]*MigInformation, 0),
-			GceService:     gceService,
-			instancesCache: make(map[GceRef]Mig),
-			machinesCache: map[MachineTypeKey]*gce.MachineType{
-				{"us-central1-b", "n1-standard-1"}: {GuestCpus: 1, MemoryMb: 1},
-				{"us-central1-c", "n1-standard-1"}: {GuestCpus: 1, MemoryMb: 1},
-				{"us-central1-f", "n1-standard-1"}: {GuestCpus: 1, MemoryMb: 1},
-			},
-			migTargetSizeCache: map[GceRef]int64{},
+	cache := &GceCache{
+		migs:                make(map[GceRef]Mig),
+		GceService:          gceService,
+		instanceRefToMigRef: make(map[GceRef]GceRef),
+		machinesCache: map[MachineTypeKey]*gce.MachineType{
+			{"us-central1-b", "n1-standard-1"}: {GuestCpus: 1, MemoryMb: 1},
+			{"us-central1-c", "n1-standard-1"}: {GuestCpus: 1, MemoryMb: 1},
+			{"us-central1-f", "n1-standard-1"}: {GuestCpus: 1, MemoryMb: 1},
 		},
-		GceService:           gceService,
-		projectId:            projectId,
-		regional:             regional,
-		templates:            &GceTemplateBuilder{},
-		explicitlyConfigured: make(map[GceRef]bool),
+		migTargetSizeCache:     map[GceRef]int64{},
+		instanceTemplatesCache: map[GceRef]*gce.InstanceTemplate{},
+		migBaseNameCache:       map[GceRef]string{},
+	}
+	manager := &gceManagerImpl{
+		cache:                        cache,
+		migTargetSizesProvider:       NewCachingMigTargetSizesProvider(cache, gceService, projectId),
+		migInstanceTemplatesProvider: NewCachingMigInstanceTemplatesProvider(cache, gceService),
+		GceService:                   gceService,
+		projectId:                    projectId,
+		regional:                     regional,
+		templates:                    &GceTemplateBuilder{},
+		explicitlyConfigured:         make(map[GceRef]bool),
 	}
 	if regional {
 		manager.location = region
@@ -294,14 +363,6 @@ func newTestGceManager(t *testing.T, testServerURL string, regional bool) *gceMa
 	return manager
 }
 
-func validateMig(t *testing.T, mig Mig, zone string, name string, minSize int, maxSize int) {
-	assert.Equal(t, name, mig.GceRef().Name)
-	assert.Equal(t, zone, mig.GceRef().Zone)
-	assert.Equal(t, projectId, mig.GceRef().Project)
-	assert.Equal(t, minSize, mig.MinSize())
-	assert.Equal(t, maxSize, mig.MaxSize())
-}
-
 const deleteInstancesResponse = `{
   "kind": "compute#operation",
   "id": "8554136016090105726",
@@ -337,10 +398,10 @@ const deleteInstancesOperationResponse = `
   "selfLink": "https://www.googleapis.com/compute/v1/projects/project1/zones/us-central1-a/operations/operation-1505802641136-55984ff86d980-a99e8c2b-0c8aaaaa"
 }`
 
-func setupTestDefaultPool(manager *gceManagerImpl) {
+func setupTestDefaultPool(manager *gceManagerImpl, setupBaseName bool) *gceMig {
 	mig := &gceMig{
 		gceRef: GceRef{
-			Name:    defaultPoolMig,
+			Name:    defaultPoolMigName,
 			Zone:    zoneB,
 			Project: projectId,
 		},
@@ -348,13 +409,17 @@ func setupTestDefaultPool(manager *gceManagerImpl) {
 		minSize:    1,
 		maxSize:    11,
 	}
-	manager.cache.migs = append(manager.cache.migs, &MigInformation{Config: mig})
+	manager.cache.migs[mig.GceRef()] = mig
+	if setupBaseName {
+		manager.cache.migBaseNameCache[mig.GceRef()] = defaultPoolMigName
+	}
+	return mig
 }
 
-func setupTestExtraPool(manager *gceManagerImpl) {
+func setupTestExtraPool(manager *gceManagerImpl, setupBaseName bool) *gceMig {
 	mig := &gceMig{
 		gceRef: GceRef{
-			Name:    extraPoolMig,
+			Name:    extraPoolMigName,
 			Zone:    zoneB,
 			Project: projectId,
 		},
@@ -362,7 +427,29 @@ func setupTestExtraPool(manager *gceManagerImpl) {
 		minSize:    0,
 		maxSize:    1000,
 	}
-	manager.cache.migs = append(manager.cache.migs, &MigInformation{Config: mig})
+	manager.cache.migs[mig.GceRef()] = mig
+	if setupBaseName {
+		manager.cache.migBaseNameCache[mig.GceRef()] = extraPoolMigName
+	}
+	return mig
+}
+
+func setupTestExtraPool2(manager *gceManagerImpl, setupBaseName bool) *gceMig {
+	mig := &gceMig{
+		gceRef: GceRef{
+			Name:    extraPool2MigName,
+			Zone:    zoneC,
+			Project: projectId,
+		},
+		gceManager: manager,
+		minSize:    0,
+		maxSize:    1000,
+	}
+	manager.cache.migs[mig.GceRef()] = mig
+	if setupBaseName {
+		manager.cache.migBaseNameCache[mig.GceRef()] = extraPool2MigName
+	}
+	return mig
 }
 
 func TestDeleteInstances(t *testing.T) {
@@ -370,18 +457,19 @@ func TestDeleteInstances(t *testing.T) {
 	defer server.Close()
 	g := newTestGceManager(t, server.URL, false)
 
-	setupTestDefaultPool(g)
-	setupTestExtraPool(g)
+	setupTestDefaultPool(g, false)
+	setupTestExtraPool(g, true)
 
-	// Test DeleteInstance function.
+	// Get basename for defaultPool
 	server.On("handle", "/project1/zones/us-central1-b/instanceGroupManagers/gke-cluster-1-default-pool").Return(buildDefaultInstanceGroupManagerResponse(zoneB)).Once()
+
+	// Regenerate instances for defaultPool
 	server.On("handle", "/project1/zones/us-central1-b/instanceGroupManagers/gke-cluster-1-default-pool/listManagedInstances").Return(buildFourRunningInstancesOnDefaultMigManagedInstancesResponse(zoneB)).Once()
-	server.On("handle", "/project1/zones/us-central1-b/instanceGroupManagers/gke-cluster-1-extra-pool-323233232").Return(buildDefaultInstanceGroupManagerResponse(zoneB)).Once()
-	server.On("handle", "/project1/zones/us-central1-b/instanceGroupManagers/gke-cluster-1-extra-pool-323233232/listManagedInstances").Return(buildOneRunningInstanceOnExtraPoolMigManagedInstancesResponse(zoneB)).Once()
+
 	server.On("handle", "/project1/zones/us-central1-b/instanceGroupManagers/gke-cluster-1-default-pool/deleteInstances").Return(deleteInstancesResponse).Once()
 	server.On("handle", "/project1/zones/us-central1-b/operations/operation-1505802641136-55984ff86d980-a99e8c2b-0c8aaaaa").Return(deleteInstancesOperationResponse).Once()
 
-	instances := []*GceRef{
+	instances := []GceRef{
 		{
 			Project: projectId,
 			Zone:    zoneB,
@@ -398,8 +486,11 @@ func TestDeleteInstances(t *testing.T) {
 	assert.NoError(t, err)
 	mock.AssertExpectationsForObjects(t, server)
 
+	// Regenerate instances for extraPool (no basename call because it is already in cache)
+	server.On("handle", "/project1/zones/us-central1-b/instanceGroupManagers/gke-cluster-1-extra-pool-323233232/listManagedInstances").Return(buildOneRunningInstanceOnExtraPoolMigManagedInstancesResponse(zoneB)).Once()
+
 	// Fail on deleting instances from different MIGs.
-	instances = []*GceRef{
+	instances = []GceRef{
 		{
 			Project: projectId,
 			Zone:    zoneB,
@@ -418,66 +509,7 @@ func TestDeleteInstances(t *testing.T) {
 	mock.AssertExpectationsForObjects(t, server)
 }
 
-func TestGetMigSizeWithCache(t *testing.T) {
-	server := NewHttpServerMock()
-	defer server.Close()
-	g := newTestGceManager(t, server.URL, false)
-
-	server.On("handle", "/project1/zones/us-central1-b/instanceGroupManagers/extra-pool-323233232").Return(instanceGroupManagerResponseTemplate).Once()
-
-	mig := &gceMig{
-		gceRef: GceRef{
-			Project: projectId,
-			Zone:    zoneB,
-			Name:    "extra-pool-323233232",
-		},
-		gceManager: g,
-		minSize:    0,
-		maxSize:    1000,
-	}
-	size, err := g.GetMigSize(mig)
-	assert.NoError(t, err)
-	assert.Equal(t, int64(3), size)
-
-	size, err = g.GetMigSize(mig)
-	assert.Equal(t, int64(3), size)
-	mock.AssertExpectationsForObjects(t, server)
-}
-
-func TestGetAndSetMigSizeWithCache(t *testing.T) {
-	server := NewHttpServerMock()
-	defer server.Close()
-	g := newTestGceManager(t, server.URL, false)
-
-	server.On("handle", "/project1/zones/us-central1-b/instanceGroupManagers/extra-pool-323233232").Return(instanceGroupManagerResponseTemplate).Once()
-	server.On("handle", "/project1/zones/us-central1-b/instanceGroupManagers/extra-pool-323233232/resize").Return(setMigSizeResponse).Once()
-	server.On("handle", "/project1/zones/us-central1-b/operations/operation-1505739408819-5597646964339-eb839c88-28805931").Return(setMigSizeOperationResponse).Once()
-	server.On("handle", "/project1/zones/us-central1-b/instanceGroupManagers/extra-pool-323233232").Return(instanceGroupManagerTargetSize4ResponseTemplate).Once()
-
-	mig := &gceMig{
-		gceRef: GceRef{
-			Project: projectId,
-			Zone:    zoneB,
-			Name:    "extra-pool-323233232",
-		},
-		gceManager: g,
-		minSize:    0,
-		maxSize:    1000,
-	}
-	size, err := g.GetMigSize(mig)
-	assert.NoError(t, err)
-	assert.Equal(t, int64(3), size)
-
-	err = g.SetMigSize(mig, 4)
-	assert.NoError(t, err)
-
-	size, err = g.GetMigSize(mig)
-	assert.Equal(t, int64(4), size)
-
-	size, err = g.GetMigSize(mig)
-	assert.Equal(t, int64(4), size)
-	mock.AssertExpectationsForObjects(t, server)
-}
+// TODO; make Test*MigSize tests use MigTargetSizesProvider mock and move mocking API server to tests of cachingMigTargetSizesProvider
 
 const setMigSizeResponse = `{
   "kind": "compute#operation",
@@ -513,26 +545,129 @@ const setMigSizeOperationResponse = `{
   "selfLink": "https://www.googleapis.com/compute/v1/projects/project1/zones/us-central1-a/operations/operation-1505739408819-5597646964339-eb839c88-28805931"
 }`
 
-func TestSetMigSize(t *testing.T) {
+func TestGetAndSetMigSize(t *testing.T) {
 	server := NewHttpServerMock()
 	defer server.Close()
 	g := newTestGceManager(t, server.URL, false)
 
-	server.On("handle", "/project1/zones/us-central1-b/instanceGroupManagers/extra-pool-323233232/resize").Return(setMigSizeResponse).Once()
-	server.On("handle", "/project1/zones/us-central1-b/operations/operation-1505739408819-5597646964339-eb839c88-28805931").Return(setMigSizeOperationResponse).Once()
+	extraPoolMig := setupTestExtraPool(g, true)
+	defaultPoolMig := setupTestDefaultPool(g, true)
 
-	mig := &gceMig{
-		gceRef: GceRef{
-			Project: projectId,
-			Zone:    zoneB,
-			Name:    "extra-pool-323233232",
-		},
-		gceManager: g,
-		minSize:    0,
-		maxSize:    1000,
-	}
-	err := g.SetMigSize(mig, 3)
+	server.On("handle", "/project1/zones/us-central1-b/instanceGroupManagers").Return(
+		buildListInstanceGroupManagersResponse(
+			buildListInstanceGroupManagersResponsePart(defaultPoolMigName, zoneB, 7),
+			buildListInstanceGroupManagersResponsePart(extraPoolMigName, zoneB, 8),
+		)).Once()
+
+	// getting size for defaultPoolMig should trigger listing all the InstanceGroupManagers
+	defaultPoolMigSize, err := g.GetMigSize(defaultPoolMig)
 	assert.NoError(t, err)
+	assert.Equal(t, int64(7), defaultPoolMigSize)
+	mock.AssertExpectationsForObjects(t, server)
+
+	// extra queries for defaultPoolMig and extraPoolMig should not result in any extra API calls
+	defaultPoolMigSize, err = g.GetMigSize(defaultPoolMig)
+	assert.NoError(t, err)
+	assert.Equal(t, int64(7), defaultPoolMigSize)
+
+	extraPoolMigSize, err := g.GetMigSize(extraPoolMig)
+	assert.NoError(t, err)
+	assert.Equal(t, int64(8), extraPoolMigSize)
+	mock.AssertExpectationsForObjects(t, server)
+
+	// set target size for extraPoolMig; will require resize API call and API call for polling for resize operation
+	server.On("handle", fmt.Sprintf("/project1/zones/us-central1-b/instanceGroupManagers/%s/resize", extraPoolMigName)).Return(setMigSizeResponse).Once()
+	server.On("handle", "/project1/zones/us-central1-b/operations/operation-1505739408819-5597646964339-eb839c88-28805931").Return(setMigSizeOperationResponse).Once()
+	err = g.SetMigSize(extraPoolMig, 4)
+	assert.NoError(t, err)
+	mock.AssertExpectationsForObjects(t, server)
+
+	// query for size of resized extraPoolMig; no extra API calls
+	extraPoolMigSize, err = g.GetMigSize(extraPoolMig)
+	assert.NoError(t, err)
+	assert.Equal(t, int64(4), extraPoolMigSize)
+	mock.AssertExpectationsForObjects(t, server)
+
+	// register another pool: extraPool2; pool uses mig in different zone
+	extraPool2Mig := setupTestExtraPool2(g, true)
+
+	// query for size of resized extraPool2Mig; execting API call refreshing target sizes
+	server.On("handle", "/project1/zones/us-central1-b/instanceGroupManagers").Return(
+		buildListInstanceGroupManagersResponse(
+			buildListInstanceGroupManagersResponsePart(defaultPoolMigName, zoneB, 7),
+			buildListInstanceGroupManagersResponsePart(extraPoolMigName, zoneB, 8),
+		)).Once()
+	server.On("handle", "/project1/zones/us-central1-c/instanceGroupManagers").Return(
+		buildListInstanceGroupManagersResponse(
+			buildListInstanceGroupManagersResponsePart(extraPool2MigName, zoneC, 9)),
+	).Once()
+
+	extraPool2MigSize, err := g.GetMigSize(extraPool2Mig)
+	assert.NoError(t, err)
+	assert.Equal(t, int64(9), extraPool2MigSize)
+	mock.AssertExpectationsForObjects(t, server)
+
+	// another query for size of extraPool2Mig will not result in any API calls
+	extraPool2MigSize, err = g.GetMigSize(extraPool2Mig)
+	assert.NoError(t, err)
+	assert.Equal(t, int64(9), extraPool2MigSize)
+	mock.AssertExpectationsForObjects(t, server)
+
+	// let's invalidate target size cache
+	// TODO we should probably call g.Refresh here but that imples more API calls. Leaving just partial cache invalidation for now
+	g.cache.InvalidateAllMigTargetSizes()
+
+	// now if w query size of any mig whole cache should be refreshed by listing InstanceGroupManagers; we expect two calls
+	// for zoneB and zoneC
+	server.On("handle", "/project1/zones/us-central1-b/instanceGroupManagers").Return(
+		buildListInstanceGroupManagersResponse(
+			buildListInstanceGroupManagersResponsePart(defaultPoolMigName, zoneB, 7),
+			buildListInstanceGroupManagersResponsePart(extraPoolMigName, zoneB, 8),
+		)).Once()
+	server.On("handle", "/project1/zones/us-central1-c/instanceGroupManagers").Return(
+		buildListInstanceGroupManagersResponse(
+			buildListInstanceGroupManagersResponsePart(extraPool2MigName, zoneC, 9),
+		)).Once()
+
+	extraPool2MigSize, err = g.GetMigSize(extraPool2Mig)
+	assert.NoError(t, err)
+	assert.Equal(t, int64(9), extraPool2MigSize)
+	mock.AssertExpectationsForObjects(t, server)
+}
+
+func TestGetMigSizeListCallFails(t *testing.T) {
+	server := NewHttpServerMock()
+	defer server.Close()
+	g := newTestGceManager(t, server.URL, false)
+
+	extraPoolMig := setupTestExtraPool(g, true)
+	defaultPoolMig := setupTestDefaultPool(g, true)
+
+	server.On("handle", "/project1/zones/us-central1-b/instanceGroupManagers").Return("bad_response").Once()
+	server.On("handle", fmt.Sprintf("/project1/zones/us-central1-b/instanceGroupManagers/%s", defaultPoolMigName)).Return(buildInstanceGroupManagerResponse(zoneB, defaultPoolMigName, 7)).Once()
+
+	// getting size for defaultPoolMig should trigger listing all the InstanceGroupManagers
+	defaultPoolMigSize, err := g.GetMigSize(defaultPoolMig)
+	assert.NoError(t, err)
+	assert.Equal(t, int64(7), defaultPoolMigSize)
+	mock.AssertExpectationsForObjects(t, server)
+
+	// extra queries for defaultPoolMig and extraPoolMig should not result in any extra API calls
+	defaultPoolMigSize, err = g.GetMigSize(defaultPoolMig)
+	assert.NoError(t, err)
+	assert.Equal(t, int64(7), defaultPoolMigSize)
+
+	// Querying another mig will yet again try to list all migs
+	server.On("handle", "/project1/zones/us-central1-b/instanceGroupManagers").Return(
+		buildListInstanceGroupManagersResponse(
+			buildListInstanceGroupManagersResponsePart(defaultPoolMigName, zoneB, 7),
+			buildListInstanceGroupManagersResponsePart(extraPoolMigName, zoneB, 8),
+		)).Once()
+
+	// getting size for defaultPoolMig should trigger listing all the InstanceGroupManagers
+	extraPoolMigSize, err := g.GetMigSize(extraPoolMig)
+	assert.NoError(t, err)
+	assert.Equal(t, int64(8), extraPoolMigSize)
 	mock.AssertExpectationsForObjects(t, server)
 }
 
@@ -541,11 +676,12 @@ func TestGetMigForInstance(t *testing.T) {
 	defer server.Close()
 	g := newTestGceManager(t, server.URL, false)
 
-	setupTestDefaultPool(g)
+	setupTestDefaultPool(g, false)
+	g.cache.InvalidateAllMigBasenames()
 
 	server.On("handle", "/project1/zones/us-central1-b/instanceGroupManagers/gke-cluster-1-default-pool").Return(buildDefaultInstanceGroupManagerResponse(zoneB)).Once()
 	server.On("handle", "/project1/zones/us-central1-b/instanceGroupManagers/gke-cluster-1-default-pool/listManagedInstances").Return(buildFourRunningInstancesOnDefaultMigManagedInstancesResponse(zoneB)).Once()
-	gceRef := &GceRef{
+	gceRef := GceRef{
 		Project: projectId,
 		Zone:    zoneB,
 		Name:    "gke-cluster-1-default-pool-f7607aac-f1hm",
@@ -771,7 +907,7 @@ func TestGetMigNodesComplex(t *testing.T) {
 			buildManagedInstanceWithCurrentActionAndErrorResponsePart("europe-west1-b", "instance-creating-other-error", "CREATING", "SOME_ERROR", "Ojojojoj!"),
 			cloudprovider.InstanceCreating,
 			cloudprovider.OtherErrorClass,
-			"",
+			ErrorCodeOther,
 			"Ojojojoj!",
 		},
 		{
@@ -931,14 +1067,13 @@ func TestFetchAutoMigsZonal(t *testing.T) {
 	defer server.Close()
 
 	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroups").Return(buildListInstanceGroupsResponse(zoneB, gceMigA, gceMigB)).Once()
-	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigA).Return(buildInstanceGroupManagerResponse(zoneB, gceMigA)).Once()
-	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigB).Return(buildInstanceGroupManagerResponse(zoneB, gceMigB)).Once()
+	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigA).Return(buildInstanceGroupManagerResponse(zoneB, gceMigA, 3)).Once()
+	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigB).Return(buildInstanceGroupManagerResponse(zoneB, gceMigB, 3)).Once()
 
+	// Regenerate instance cache
 	server.On("handle", "/project1/global/instanceTemplates/"+gceMigA).Return(instanceTemplate).Once()
 	server.On("handle", "/project1/global/instanceTemplates/"+gceMigB).Return(instanceTemplate).Once()
-	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigA).Return(buildInstanceGroupManagerResponse(zoneB, gceMigA)).Once()
 	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigA+"/listManagedInstances").Return(buildFourRunningInstancesManagedInstancesResponse(zoneB, gceMigA)).Once()
-	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigB).Return(buildInstanceGroupManagerResponse(zoneB, gceMigB)).Once()
 	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigB+"/listManagedInstances").Return(buildOneRunningInstanceManagedInstancesResponse(zoneB, gceMigB)).Once()
 
 	regional := false
@@ -953,8 +1088,8 @@ func TestFetchAutoMigsZonal(t *testing.T) {
 
 	migs := g.GetMigs()
 	assert.Equal(t, 2, len(migs))
-	validateMig(t, migs[0].Config, zoneB, gceMigA, min, max)
-	validateMig(t, migs[1].Config, zoneB, gceMigB, min, max)
+	validateMigExists(t, migs, zoneB, gceMigA, min, max)
+	validateMigExists(t, migs, zoneB, gceMigB, min, max)
 	mock.AssertExpectationsForObjects(t, server)
 }
 
@@ -963,15 +1098,14 @@ func TestFetchAutoMigsUnregistersMissingMigs(t *testing.T) {
 	defer server.Close()
 
 	// Register explicit instance group
-	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigA).Return(buildInstanceGroupManagerResponse(zoneB, gceMigA)).Once()
+	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigA).Return(buildInstanceGroupManagerResponse(zoneB, gceMigA, 3)).Once()
 	server.On("handle", "/project1/global/instanceTemplates/"+gceMigA).Return(instanceTemplate).Once()
 
 	// Regenerate cache for explicit instance group
-	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigA).Return(buildInstanceGroupManagerResponse(zoneB, gceMigA)).Twice()
 	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigA+"/listManagedInstances").Return(buildFourRunningInstancesManagedInstancesResponse(zoneB, gceMigA)).Twice()
 
 	// Register 'previously autodetected' instance group
-	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigB).Return(buildInstanceGroupManagerResponse(zoneB, gceMigB)).Once()
+	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigB).Return(buildInstanceGroupManagerResponse(zoneB, gceMigB, 3)).Once()
 	server.On("handle", "/project1/global/instanceTemplates/"+gceMigB).Return(instanceTemplate).Once()
 
 	regional := false
@@ -996,7 +1130,7 @@ func TestFetchAutoMigsUnregistersMissingMigs(t *testing.T) {
 
 	migs := g.GetMigs()
 	assert.Equal(t, 1, len(migs))
-	validateMig(t, migs[0].Config, zoneB, gceMigA, minA, maxA)
+	validateMigExists(t, migs, zoneB, gceMigA, minA, maxA)
 	mock.AssertExpectationsForObjects(t, server)
 }
 
@@ -1006,14 +1140,13 @@ func TestFetchAutoMigsRegional(t *testing.T) {
 
 	server.On("handle", "/project1/regions/us-central1").Return(getRegionResponse).Once()
 	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroups").Return(buildListInstanceGroupsResponse(zoneB, gceMigA, gceMigB)).Once()
-	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigA).Return(buildInstanceGroupManagerResponse(zoneB, gceMigA)).Once()
-	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigB).Return(buildInstanceGroupManagerResponse(zoneB, gceMigB)).Once()
+	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigA).Return(buildInstanceGroupManagerResponse(zoneB, gceMigA, 3)).Once()
+	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigB).Return(buildInstanceGroupManagerResponse(zoneB, gceMigB, 3)).Once()
 
+	// Regenerate instance cache
 	server.On("handle", "/project1/global/instanceTemplates/"+gceMigA).Return(instanceTemplate).Once()
 	server.On("handle", "/project1/global/instanceTemplates/"+gceMigB).Return(instanceTemplate).Once()
-	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigA).Return(buildInstanceGroupManagerResponse(zoneB, gceMigA)).Once()
 	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigA+"/listManagedInstances").Return(buildFourRunningInstancesManagedInstancesResponse(zoneB, gceMigA)).Once()
-	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigB).Return(buildInstanceGroupManagerResponse(gceMigB, zoneB)).Once()
 	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigB+"/listManagedInstances").Return(buildOneRunningInstanceManagedInstancesResponse(zoneB, gceMigB)).Once()
 
 	regional := true
@@ -1028,8 +1161,8 @@ func TestFetchAutoMigsRegional(t *testing.T) {
 
 	migs := g.GetMigs()
 	assert.Equal(t, 2, len(migs))
-	validateMig(t, migs[0].Config, zoneB, gceMigA, min, max)
-	validateMig(t, migs[1].Config, zoneB, gceMigB, min, max)
+	validateMigExists(t, migs, zoneB, gceMigA, min, max)
+	validateMigExists(t, migs, zoneB, gceMigB, min, max)
 	mock.AssertExpectationsForObjects(t, server)
 }
 
@@ -1037,15 +1170,13 @@ func TestFetchExplicitMigs(t *testing.T) {
 	server := NewHttpServerMock()
 	defer server.Close()
 
-	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigA).Return(buildInstanceGroupManagerResponse(zoneB, gceMigA)).Once()
-	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigB).Return(buildInstanceGroupManagerResponse(zoneB, gceMigB)).Once()
+	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigA).Return(buildInstanceGroupManagerResponse(zoneB, gceMigA, 3)).Once()
+	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigB).Return(buildInstanceGroupManagerResponse(zoneB, gceMigB, 3)).Once()
 
 	server.On("handle", "/project1/global/instanceTemplates/"+gceMigA).Return(instanceTemplate).Once()
 	server.On("handle", "/project1/global/instanceTemplates/"+gceMigB).Return(instanceTemplate).Once()
 
-	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigA).Return(buildInstanceGroupManagerResponse(zoneB, gceMigA)).Once()
 	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigA+"/listManagedInstances").Return(buildFourRunningInstancesManagedInstancesResponse(zoneB, gceMigA)).Once()
-	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigB).Return(buildInstanceGroupManagerResponse(zoneB, gceMigB)).Once()
 	server.On("handle", "/project1/zones/"+zoneB+"/instanceGroupManagers/"+gceMigB+"/listManagedInstances").Return(buildOneRunningInstanceManagedInstancesResponse(zoneB, gceMigB)).Once()
 
 	regional := false
@@ -1062,8 +1193,8 @@ func TestFetchExplicitMigs(t *testing.T) {
 
 	migs := g.GetMigs()
 	assert.Equal(t, 2, len(migs))
-	validateMig(t, migs[0].Config, zoneB, gceMigA, minA, maxA)
-	validateMig(t, migs[1].Config, zoneB, gceMigB, minB, maxB)
+	validateMigExists(t, migs, zoneB, gceMigA, minA, maxA)
+	validateMigExists(t, migs, zoneB, gceMigB, minB, maxB)
 	mock.AssertExpectationsForObjects(t, server)
 }
 
@@ -1200,3 +1331,23 @@ func TestParseCustomMachineType(t *testing.T) {
 	cpu, mem, err = parseCustomMachineType("other-2-2816")
 	assert.Error(t, err)
 }
+
+func validateMigExists(t *testing.T, migs []Mig, zone string, name string, minSize int, maxSize int) {
+	ref := GceRef{
+		projectId,
+		zone,
+		name,
+	}
+	for _, mig := range migs {
+		if mig.GceRef() == ref {
+			assert.Equal(t, minSize, mig.MinSize())
+			assert.Equal(t, maxSize, mig.MaxSize())
+			return
+		}
+	}
+	allRefs := []GceRef{}
+	for _, mig := range migs {
+		allRefs = append(allRefs, mig.GceRef())
+	}
+	assert.Failf(t, "Mig not found", "Mig %v not found among %v", ref, allRefs)
+}
